@@ -53,8 +53,8 @@ namespace Muine.PluginLib
 		void Next ();
 		void Previous ();
 
-		void PlaySong ();
-		void PlayAlbum ();
+		void PlaySong (uint time);
+		void PlayAlbum (uint time);
 
 		ISong [] Playlist {
 			get;
@@ -77,8 +77,9 @@ namespace Muine.PluginLib
 
 		bool WindowVisible {
 			get;
-			set;
 		}
+
+                void SetWindowVisible (bool visible, uint time);
 	
 		Gtk.UIManager UIManager {
 			get;
@@ -95,6 +96,20 @@ namespace Muine.PluginLib
 			get;
 		}
 
+		string [] WatchedFolders {
+                        set;
+			get;
+		}
+
+		void AddFolder (string folder);
+		void RemoveFolder (string folder);
+
+		ISong AddSong (string path);
+                void SyncSong (string path);
+                void SyncSong (ISong song);
+		void RemoveSong (string path);
+		void RemoveSong (ISong song);
+
 		event SongChangedEventHandler SongChangedEvent;
 	
 		event StateChangedEventHandler StateChangedEvent;
@@ -104,6 +119,8 @@ namespace Muine.PluginLib
 		event GenericEventHandler PlaylistChangedEvent;
 
 		event GenericEventHandler SelectionChangedEvent;
+
+		event GenericEventHandler WatchedFoldersChangedEvent;
 	}
 
 	public delegate void SongChangedEventHandler (ISong song);
