@@ -34,28 +34,41 @@ namespace Muine
 		private string gconf_path;
 
 		// Constructor
-		public FileSelector (string title, FileChooserAction action, string gcp) : base (title, Global.Playlist, action, "gnome-vfs")
+		/// <summary>
+		///	Creates a new <see cref="FileSelector" />.
+		/// </summary>
+		/// <param name="title">
+		///	The window title.
+		/// </param>
+		/// <param name="action">
+		///	The <see cref="FileChooserAction" /> to do.
+		/// </param>
+		/// <param name="gconf_path">
+		///	The GConf path which holds the starting directory.
+		/// </param>
+		public FileSelector (string title, FileChooserAction action, string gconf_path) 
+		: base (title, Global.Playlist, action, "gnome-vfs")
 		{
-			LocalOnly = false;
+			this.gconf_path = gconf_path;
 
-			AddButton (Stock.Cancel, ResponseType.Cancel);
+			base.LocalOnly = false;
+
+			base.AddButton (Stock.Cancel, ResponseType.Cancel);
 
 			switch (action) {
 			case FileChooserAction.Open:
-				AddButton (Stock.Open, ResponseType.Ok);
+				base.AddButton (Stock.Open, ResponseType.Ok);
 				break;
 			
 			case FileChooserAction.Save:
-				AddButton (Stock.Save, ResponseType.Ok);
+				base.AddButton (Stock.Save, ResponseType.Ok);
 				break;
 			
 			default:
 				break;
 			}
 			
-			DefaultResponse = ResponseType.Ok;
-
-			gconf_path = gcp;
+			base.DefaultResponse = ResponseType.Ok;
 
 			string start_dir = (string) Config.Get (gconf_path, GConfDefaultStartDir);
 
@@ -69,10 +82,25 @@ namespace Muine
 
 		// Handlers
 		// Handlers :: OnResponse
+		/// <summary>
+		///	Handler called when a response has been chosen.
+		/// </summary>
+		/// <remarks>
+		///	If the response is "Ok", then save the current
+		///	directory to GConf. Otherwise, just close
+		/// </remarks>
+		/// <param name="o">
+		///	The calling object.
+		/// </param>
+		/// <param name="args">
+		///	The <see cref="ResponseArgs" />.
+		/// </param>
 		private void OnResponse (object o, ResponseArgs args)
 		{
-			if (args.ResponseId == ResponseType.Ok)
-				Config.Set (gconf_path, CurrentFolderUri);
+			if (args.ResponseId != ResponseType.Ok)
+				return;
+
+			Config.Set (gconf_path, CurrentFolderUri);
 		}
 	}
 }
