@@ -22,7 +22,7 @@ using System;
 using System.Collections;
 using System.IO;
 using Gtk;
-using Mono.Posix;
+using Mono.Unix;
 
 namespace Muine
 {
@@ -91,7 +91,7 @@ namespace Muine
 
 			// Check if we have any Directories to add
 			if (new_dinfos.Count > 0)
-			        Global.DB.AddFolders (new_dinfos);
+			        Global.DB.AddWatchedFolders (new_dinfos);
 
                         base.Destroy ();
 		}
