@@ -25,7 +25,7 @@ using Gtk;
 using GLib;
 using Gnome.Vfs;
 
-using Mono.Posix;
+using Mono.Unix;
 
 using Muine.PluginLib;
 
@@ -143,6 +143,9 @@ namespace Muine
 		// Events :: SelectionChangedEvent (IPlayer)
 		public event GenericEventHandler SelectionChangedEvent;
 
+		// Events :: WatchedFoldersChangedEvent (IPlayer)
+		public event GenericEventHandler WatchedFoldersChangedEvent;
+
 		// Delegates
 		private delegate void PlaylistForeachFunc (Song song, bool playing, object user_data);
 
@@ -158,17 +161,8 @@ namespace Muine
 		
 		// Widgets :: Containers
 		[Glade.Widget] private Container volume_button_container;
-		[Glade.Widget] private Container title_label_container  ;
-		[Glade.Widget] private Container artist_label_container ;
 		[Glade.Widget] private Container cover_image_container  ;
 
-		// Widgets :: Images
-		[Glade.Widget] private Image toggle_play_image;
-		[Glade.Widget] private Image previous_image   ;
-		[Glade.Widget] private Image next_image       ;
-		[Glade.Widget] private Image add_song_image   ;
-		[Glade.Widget] private Image add_album_image  ;
-
 		// Widgets :: Toolbar
 		[Glade.Widget] private ToggleButton toggle_play_button;
 		[Glade.Widget] private Button       previous_button   ;
@@ -179,11 +173,10 @@ namespace Muine
 		private VolumeButton volume_button;
 
 		// Widgets :: Player
+		[Glade.Widget] private Label song_label;
 		[Glade.Widget] private Label time_label;
 
 		private CoverImage       cover_image ;
-		private EllipsizingLabel title_label ;
-		private EllipsizingLabel artist_label;
 
 		// Widgets :: Playlist
 		[Glade.Widget] private Label    playlist_label          ;
@@ -263,8 +256,9 @@ namespace Muine
 			SetupPlaylist ();
 
 			// Connect to song database signals
-			Global.DB.SongChanged += new SongDatabase.SongChangedHandler (OnSongChanged);
-			Global.DB.SongRemoved += new SongDatabase.SongRemovedHandler (OnSongRemoved);
+			Global.DB.SongChanged           += new SongDatabase.SongChangedHandler (OnSongChanged);
+			Global.DB.SongRemoved           += new SongDatabase.SongRemovedHandler (OnSongRemoved);
+			Global.DB.WatchedFoldersChanged += new SongDatabase.WatchedFoldersChangedHandler (OnWatchedFoldersChanged);
 
 			// Make sure the interface is up to date
 			SelectionChanged ();
@@ -388,25 +382,14 @@ namespace Muine
 			get { return busy_level; }
 		}
 
-		// Properties :: WindowVisible (set; get;) (IPlayer)
+                // Properties :: WatchedFolders (set; get;) (IPlayer)
+ 		public string [] WatchedFolders {
+                        set { Global.DB.WatchedFolders = value; }
+ 			get { return Global.DB.WatchedFolders; }
+ 		}
+
+		// Properties :: WindowVisible (get;) (IPlayer)
 		public bool WindowVisible {
-			set {
-				window_visible = value;
-
-				if (window_visible) {
-					if (!Visible && last_x >= 0 && last_y >= 0)
-						Move (last_x, last_y);
-
-					Present ();
-
-				} else {
-					GetPosition (out last_x, out last_y);
-					Visible = false;
-				}
-
-				UpdateWindowVisibilityUI ();
-			}
-
 			get { return window_visible; }
 		}
 
@@ -431,7 +414,7 @@ namespace Muine
 		public void RestorePlaylist ()
 		{
 			// Load last playlist
-			if (!File.Exists (FileUtils.PlaylistFile))
+			if (!System.IO.File.Exists (FileUtils.PlaylistFile))
 				return;
 
 			OpenPlaylistInternal (FileUtils.PlaylistFile,
@@ -447,7 +430,7 @@ namespace Muine
 				SongChanged (true); // make sure the UI is up to date
 
 			RestoreState ();
-			WindowVisible = true;
+                        SetWindowVisible (true, 0);
 		}
 
 		// Methods :: Public :: Quit (IPlayer)
@@ -477,7 +460,7 @@ namespace Muine
 		// Methods :: Public :: PlayFile (IPlayer)
 		public void PlayFile (string file)
 		{
-			Song song = GetSingleSong (file);
+			Song song = GetSingleSong (file, false);
 
 			if (song == null)
 				return;
@@ -491,7 +474,7 @@ namespace Muine
 		// Methods :: Public :: QueueFile (IPlayer)
 		public void QueueFile (string file)
 		{
-			Song song = GetSingleSong (file);
+			Song song = GetSingleSong (file, false);
 
 			if (song == null)
 				return;
@@ -562,7 +545,7 @@ namespace Muine
 		}
 
 		// Methods :: Public :: PlaySong (IPlayer)
-		public void PlaySong ()
+		public void PlaySong (uint time)
 		{
 			if (add_song_window == null) {
 				add_song_window = new AddSongWindow ();
@@ -572,11 +555,11 @@ namespace Muine
 			}
 
 			AddChildWindowIfVisible (add_song_window);
-			add_song_window.Run ();
+			add_song_window.Run (time);
 		}
 
 		// Methods :: Public :: PlayAlbum (IPlayer)
-		public void PlayAlbum ()
+		public void PlayAlbum (uint time)
 		{
 			if (add_album_window == null) {
 				add_album_window = new AddAlbumWindow ();
@@ -586,7 +569,83 @@ namespace Muine
 			}
 
 			AddChildWindowIfVisible (add_album_window);
-			add_album_window.Run ();
+			add_album_window.Run (time);
+		}
+
+                // Methods :: Public :: SetWindowVisible (IPlayer)
+                public void SetWindowVisible (bool visible, uint time)
+                {
+			window_visible = visible;
+
+			if (window_visible) {
+                                bool was_visible = Visible;
+                                
+				if (!was_visible && last_x >= 0 && last_y >= 0)
+					Move (last_x, last_y);
+
+			        Show ();
+
+                                if (was_visible)
+                                        GdkWindow.Focus (time);
+
+			} else {
+				GetPosition (out last_x, out last_y);
+				Visible = false;
+			}
+                                
+			UpdateWindowVisibilityUI ();
+                }
+
+		// Methods :: Public :: AddSong (IPlayer)
+		public ISong AddSong (string path)
+		{
+			return GetSingleSong (path, true);
+		}
+
+                // Methods :: Public :: SyncSong (IPlayer)
+		public void SyncSong (string path)
+		{
+			lock (Global.DB) {
+				Song song = (Song) Global.DB.Songs [path];
+
+				if (song != null)
+					Global.DB.SyncSong (song);
+			}
+		}
+
+		// Methods :: Public :: SyncSong (IPlayer)
+		public void SyncSong (ISong song)
+		{
+			Global.DB.SyncSong ((Song) song);
+		}
+
+		// Methods :: Public :: RemoveSong (IPlayer)
+		public void RemoveSong (string path)
+		{
+			lock (Global.DB) {
+				Song song = (Song) Global.DB.Songs [path];
+
+				if (song != null)
+					Global.DB.RemoveSong (song);
+			}
+		}
+
+		// Methods :: Public :: RemoveSong (IPlayer)
+		public void RemoveSong (ISong song)
+		{
+			Global.DB.RemoveSong ((Song) song);
+		}
+
+		// Methods :: Public :: AddFolder (IPlayer)
+		public void AddFolder (string folder)
+		{
+			Global.DB.AddFolder (folder);
+		}
+
+		// Methods :: Public :: RemoveFolder (IPlayer)
+		public void RemoveFolder (string folder)
+		{
+			Global.DB.RemoveFolder (folder);
 		}
 
 		// Methods :: Public :: AddChildWindowIfVisible
@@ -606,12 +665,6 @@ namespace Muine
 			skip_to_window.Run ();
 		}
 
-		// Methods :: Public :: ToggleVisible
-		public void ToggleVisible ()
-		{
-			WindowVisible = !WindowVisible;
-		}
-
 		// Methods :: Public :: SkipBackwards		
 		public void SkipBackwards ()
 		{				
@@ -666,8 +719,7 @@ namespace Muine
 				return;
 
 			if (had_last_eos) {
-				ClearPlaylist ();
-				PlaylistChanged ();
+				Clear ();
 				return;
 			}
 
@@ -791,13 +843,6 @@ namespace Muine
 			add_song_button   .Clicked += new EventHandler (OnAddSongButtonClicked  );
 			add_album_button  .Clicked += new EventHandler (OnAddAlbumButtonClicked );
 
-			// Images
-			toggle_play_image.SetFromStock ("stock_media-play"     , IconSize.LargeToolbar);
-			previous_image   .SetFromStock ("stock_media-prev"     , IconSize.LargeToolbar);
-			next_image       .SetFromStock ("stock_media-next"     , IconSize.LargeToolbar);
-			add_song_image   .SetFromStock (Stock.Add              , IconSize.LargeToolbar);
-			add_album_image  .SetFromStock ("gnome-dev-cdrom-audio", IconSize.LargeToolbar);
-
 			// Volume
 			volume_button = new VolumeButton ();
 			volume_button_container.Add (volume_button);
@@ -871,20 +916,6 @@ namespace Muine
 			player.StateChanged     += new Player.StateChangedHandler     (OnStateChanged    );
 			player.InvalidSong      += new Player.InvalidSongHandler      (OnInvalidSong     );
 
-			// Title
-			title_label = new EllipsizingLabel ();
-			title_label.Xalign = 0.0f;
-			title_label.Visible    = true;
-			title_label.Selectable = true;
-			title_label_container.Add (title_label);
-
-			// Artist
-			artist_label = new EllipsizingLabel ();
-			artist_label.Xalign = 0.0f;
-			artist_label.Visible    = true;
-			artist_label.Selectable = true;
-			artist_label_container.Add (artist_label);
-
 			// Cover Image
 			cover_image = new CoverImage ();
 			cover_image_container.Add (cover_image);
@@ -1087,10 +1118,14 @@ namespace Muine
 		// Methods :: Private :: SongChanged
 		private void SongChanged (bool restart)
 		{
-			Song song = null;
+			SongChanged (restart, true);
+		}
 
+		// Methods :: Private :: SongChanged
+		private void SongChanged (bool restart, bool fire_signal)
+		{
 			if (playlist.Model.Playing != IntPtr.Zero) {
-				song = Song.FromHandle (playlist.Model.Playing);
+				Song song = Song.FromHandle (playlist.Model.Playing);
 
 				cover_image.Song = song;
 
@@ -1110,38 +1145,38 @@ namespace Muine
 				
 				tooltips.SetTip (cover_image, tip, null);
 
-				title_label.Markup = String.Format (
-					"<span size=\"large\" weight=\"bold\">{0}</span>",
-					StringUtils.EscapeForPango (song.Title));
+				song_label.Markup = String.Format (
+					"<span size=\"large\" weight=\"bold\">{0}</span>\n{1}",
+					StringUtils.EscapeForPango (song.Title),
+                                        StringUtils.EscapeForPango (StringUtils.JoinHumanReadable (song.Artists)));
 
-				artist_label.Text = StringUtils.JoinHumanReadable (song.Artists);
+				this.Title = String.Format (string_title_main, song.Title);
 
 				if (player.Song != song || restart)
 					player.Song = song;
 
-				this.Title = String.Format (string_title_main, song.Title);
+			        if (fire_signal && SongChangedEvent != null)
+				        SongChangedEvent (song);
 
 			} else {
 				cover_image.Song = null;
 
 				tooltips.SetTip (cover_image, null, null);
 
-				title_label.Markup = "";
-
-				artist_label.Text = "";
+				song_label.Markup = "";
 				time_label  .Text = "";
 
 				this.Title = string_program;
 
 				if (skip_to_window != null)
 					skip_to_window.Hide ();
+
+			        if (SongChangedEvent != null)
+				        SongChangedEvent (null);
 			}
 			
 			if (restart)
 				had_last_eos = false;
-
-			if (SongChangedEvent != null)
-				SongChangedEvent (song);
 		}
 
 		// Methods :: Private :: SelectionChanged
@@ -1304,7 +1339,7 @@ namespace Muine
 		}
 
 		// Methods :: Private :: GetSingleSong
-		private Song GetSingleSong (string file)
+		private Song GetSingleSong (string file, bool sync)
 		{
 			// Get Song
 			Song song = Global.DB.GetSong (file);
@@ -1312,6 +1347,8 @@ namespace Muine
 			// If we don't have it, try adding it
 			if (song == null)
 				song = AddSongToDB (file);
+                        else if (sync)
+                                Global.DB.SyncSong (song);
 
 			return song;
 		}
@@ -1650,6 +1687,13 @@ namespace Muine
 			SelectionChanged ();
 		}
 
+		// Handlers :: OnWatchedFoldersChanged
+		private void OnWatchedFoldersChanged ()
+		{
+			if (WatchedFoldersChangedEvent != null)
+				WatchedFoldersChangedEvent ();
+		}
+
 		// Handlers :: OnPlaylistPlayingChanged
 		private void OnPlaylistPlayingChanged (IntPtr playing)
 		{
@@ -1669,8 +1713,10 @@ namespace Muine
 
 				song_changed = true;
 				
+				// Use overload of SongChanged that won't fire the "SongChanged" event, since
+				// we really only want to update the pixbuf, labels, etc.
 				if (h == playlist.Model.Playing)
-					SongChanged (false);
+					SongChanged (false, false);
 
 				playlist.Model.Changed (h);
 			}
@@ -1925,7 +1971,7 @@ namespace Muine
 						continue;
 					}
 
-					Song song = GetSingleSong (finfo.FullName);
+					Song song = GetSingleSong (finfo.FullName, false);
 						
 					if (song != null) {
 						DragAddSong (song, pos);
@@ -1940,7 +1986,7 @@ namespace Muine
 				}
 
 				if (new_dinfos.Count > 0) {
-					Global.DB.AddFolders (new_dinfos);
+					Global.DB.AddWatchedFolders (new_dinfos);
 					success = true;
 				}
 
@@ -1978,13 +2024,13 @@ namespace Muine
 		// Handlers :: OnAddSongButtonClicked
 		private void OnAddSongButtonClicked (object o, EventArgs args)
 		{
-			PlaySong ();
+			PlaySong (Gtk.Global.CurrentEventTime);
 		}
 		
 		// Handlers :: OnAddAlbumButtonClicked
 		private void OnAddAlbumButtonClicked (object o, EventArgs args)
 		{
-			PlayAlbum ();
+			PlayAlbum (Gtk.Global.CurrentEventTime);
 		}
 		
 		// Handlers :: OnPlaylistLabelDragDataGet
@@ -2049,7 +2095,7 @@ namespace Muine
 			}
 
 			if (new_dinfos.Count > 0) {
-				Global.DB.AddFolders (new_dinfos);
+				Global.DB.AddWatchedFolders (new_dinfos);
 				success = true;
 			}
 
